from file_automation.local.dir.dir_process import copy_dir, rename_dir, create_dir, remove_dir_tree
from file_automation.local.file.file_process import copy_file, remove_file, rename_file, copy_specify_extension_file, \
    copy_all_file_to_dir
from file_automation.local.zip.zip_process import zip_dir, zip_file, zip_info, zip_file_info, set_zip_password, \
    read_zip_file, unzip_file, unzip_all
from file_automation.remote.google_drive.delete.delete_manager import drive_delete_file
from file_automation.remote.google_drive.dir.folder_manager import drive_add_folder
from file_automation.remote.google_drive.download.download_file import drive_download_file, \
    drive_download_file_from_folder
from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.remote.google_drive.search.search_drive import \
    drive_search_all_file, drive_search_field, drive_search_file_mimetype
from file_automation.remote.google_drive.share.share_file import \
    drive_share_file_to_anyone, drive_share_file_to_domain, drive_share_file_to_user
from file_automation.remote.google_drive.upload.upload_to_driver import \
    drive_upload_dir_to_folder, drive_upload_to_folder, drive_upload_dir_to_drive, drive_upload_to_drive
from file_automation.utils.executor.action_executor import execute_action, execute_files, add_command_to_executor
from file_automation.utils.file_process.get_dir_file_list import get_dir_files_as_list
from file_automation.utils.json.json_file import read_action_json
from file_automation.utils.project.create_project_structure import create_project_dir

__all__ = [
    "copy_file", "rename_file", "remove_file", "copy_all_file_to_dir", "copy_specify_extension_file",
    "copy_dir", "create_dir", "remove_dir_tree", "zip_dir", "zip_file", "zip_info",
    "zip_file_info", "set_zip_password", "unzip_file", "read_zip_file",
    "unzip_all", "driver_instance", "drive_search_all_file", "drive_search_field", "drive_search_file_mimetype",
    "drive_upload_dir_to_folder", "drive_upload_to_folder", "drive_upload_dir_to_drive", "drive_upload_to_drive",
    "drive_add_folder", "drive_share_file_to_anyone", "drive_share_file_to_domain", "drive_share_file_to_user",
    "drive_delete_file", "drive_download_file", "drive_download_file_from_folder", "execute_action", "execute_files",
    "add_command_to_executor", "read_action_json", "get_dir_files_as_list", "create_project_dir"
]



from typing import Union

from googleapiclient.errors import HttpError

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_add_folder(folder_name: str) -> Union[dict, None]:
    try:
        file_metadata = {
            "name": folder_name,
            "mimeType": "application/vnd.google-apps.folder"
        }
        file = driver_instance.service.files().create(
            body=file_metadata,
            fields="id"
        ).execute()
        file_automation_logger.info(
            f"Add drive folder: {folder_name}"
        )
        return file.get("id")
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None



from importlib import import_module
from importlib.util import find_spec
from inspect import getmembers, isfunction, isbuiltin, isclass
from sys import stderr

from file_automation.utils.logging.loggin_instance import file_automation_logger


class PackageManager(object):

    def __init__(self):
        self.installed_package_dict = {
        }
        self.executor = None
        self.callback_executor = None

    def check_package(self, package: str):
        """
        :param package: package to check exists or not
        :return: package if find else None
        """
        if self.installed_package_dict.get(package, None) is None:
            found_spec = find_spec(package)
            if found_spec is not None:
                try:
                    installed_package = import_module(found_spec.name)
                    self.installed_package_dict.update(
                        {found_spec.name: installed_package})
                except ModuleNotFoundError as error:
                    print(repr(error), file=stderr)
        return self.installed_package_dict.get(package, None)

    def add_package_to_executor(self, package):
        file_automation_logger.info(f"add_package_to_executor, package: {package}")
        """
        :param package: package's function will add to executor
        """
        self.add_package_to_target(
            package=package,
            target=self.executor
        )

    def add_package_to_callback_executor(self, package):
        file_automation_logger.info(f"add_package_to_callback_executor, package: {package}")
        """
        :param package: package's function will add to callback_executor
        """
        self.add_package_to_target(
            package=package,
            target=self.callback_executor
        )

    def get_member(self, package, predicate, target):
        """
        :param package: package we want to get member
        :param predicate: predicate
        :param target: which event_dict will be added
        """
        installed_package = self.check_package(package)
        if installed_package is not None and target is not None:
            for member in getmembers(installed_package, predicate):
                target.event_dict.update(
                    {str(package) + "_" + str(member[0]): member[1]})
        elif installed_package is None:
            print(repr(ModuleNotFoundError(f"Can't find package {package}")),
                  file=stderr)
        else:
            print(f"Executor error {self.executor}", file=stderr)

    def add_package_to_target(self, package, target):
        """
        :param package: package we want to get member
        :param target: which event_dict will be added
        """
        try:
            self.get_member(
                package=package,
                predicate=isfunction,
                target=target
            )
            self.get_member(
                package=package,
                predicate=isbuiltin,
                target=target
            )
            self.get_member(
                package=package,
                predicate=isfunction,
                target=target
            )
            self.get_member(
                package=package,
                predicate=isclass,
                target=target
            )
        except Exception as error:
            print(repr(error), file=stderr)


package_manager = PackageManager()



from pathlib import Path
from typing import List, Union, Optional

from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_upload_to_drive(file_path: str, file_name: str = None) -> Union[dict, None]:
    try:
        file_path = Path(file_path)
        if file_path.is_file():
            file_metadata = {
                "name": file_path.name if file_name is None else file_name,
                "mimeType": "*/*"
            }
            media = MediaFileUpload(
                file_path,
                mimetype="*/*",
                resumable=True
            )
            file_id = driver_instance.service.files().create(
                body=file_metadata,
                media_body=media,
                fields="id"
            ).execute()
            file_automation_logger.info(
                f"Upload file to drive file: {file_path}, "
                f"with name: {file_name}"
            )
            return file_id
        else:
            file_automation_logger.error(
                FileNotFoundError
            )
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_upload_to_folder(folder_id: str, file_path: str, file_name: str = None) -> Union[dict, None]:
    try:
        file_path = Path(file_path)
        if file_path.is_file():
            file_metadata = {
                "name": file_path.name if file_name is None else file_name,
                "mimeType": "*/*",
                "parents": [f"{folder_id}"]
            }
            media = MediaFileUpload(
                file_path,
                mimetype="*/*",
                resumable=True
            )
            file_id = driver_instance.service.files().create(
                body=file_metadata,
                media_body=media,
                fields="id"
            ).execute()
            file_automation_logger.info(
                f"Upload file to folder: {folder_id},"
                f"file_path: {file_path}, "
                f"with name: {file_name}"
            )
            return file_id
        else:
            file_automation_logger.error(
                FileNotFoundError
            )
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_upload_dir_to_drive(dir_path: str) -> List[Optional[set]]:
    dir_path = Path(dir_path)
    ids = list()
    if dir_path.is_dir():
        path_list = dir_path.iterdir()
        for path in path_list:
            if path.is_file():
                ids.append(drive_upload_to_drive(str(path.absolute()), path.name))
        file_automation_logger.info(
            f"Upload all file on dir: {dir_path} to drive"
        )
        return ids
    else:
        file_automation_logger.error(
            FileNotFoundError
        )


def drive_upload_dir_to_folder(folder_id: str, dir_path: str) -> List[Optional[set]]:
    dir_path = Path(dir_path)
    ids = list()
    if dir_path.is_dir():
        path_list = dir_path.iterdir()
        for path in path_list:
            if path.is_file():
                ids.append(drive_upload_to_folder(folder_id, str(path.absolute()), path.name))
        file_automation_logger.info(
            f"Upload all file on dir: {dir_path} to folder: {folder_id}"
        )
        return ids
    else:
        file_automation_logger.error(
            FileNotFoundError
        )



executor_template_1: str = \
    """from file_automation import execute_action, read_action_json

execute_action(
    read_action_json(
        r"{temp}"
    )
)
"""

executor_template_2: str = \
    """from file_automation import execute_files, get_dir_files_as_list

execute_files(
    get_dir_files_as_list(
        r"{temp}"
    )
)
"""

bad_executor_template_1: str = \
    """
# This example is primarily intended to remind users of the importance of verifying input.
from file_automation import execute_action, read_action_json
    
execute_action(
    read_action_json(
        r"{temp}"
    )
)
"""


template_keyword_1: list = [
    ["FA_create_dir", {"dir_path": "test_dir"}],
    ["FA_create_file", {"file_path": "test.txt", "content": "test"}]
]

template_keyword_2: list = [
    ["FA_remove_file", {"file_path": "text.txt"}],
    ["FA_remove_dir_tree", {"FA_remove_dir_tree": "test_dir"}]
]

bad_template_1 = [
    ["FA_add_package_to_executor", ["os"]],
    ["os_system", ["python --version"]],
    ["os_system", ["python -m pip --version"]],
]



import zipfile
from pathlib import Path
from shutil import make_archive
from typing import List

from file_automation.utils.exception.exceptions import ZIPGetWrongFileException
from file_automation.utils.logging.loggin_instance import file_automation_logger


def zip_dir(dir_we_want_to_zip: str, zip_name: str) -> None:
    make_archive(root_dir=dir_we_want_to_zip, base_name=zip_name, format="zip")
    file_automation_logger.info(f"Dir to zip: {dir_we_want_to_zip}, zip file name: {zip_name}")


def zip_file(zip_file_path: str, file: [str, List[str]]) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="w")
    if isinstance(file, str):
        file_name = Path(file)
        current_zip.write(file, file_name.name)
        file_automation_logger.info(
            f"Write file: {file_name} to zip: {current_zip}"
        )
    else:
        if isinstance(file, list):
            for writeable in file:
                file_name = Path(writeable)
                current_zip.write(writeable, file_name.name)
                file_automation_logger.info(
                    f"Write file: {writeable} to zip: {current_zip}"
                )
        else:
            file_automation_logger.error(
                repr(ZIPGetWrongFileException))
    current_zip.close()


def read_zip_file(zip_file_path: str, file_name: str, password: [str, None] = None) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="r")
    with current_zip.open(name=file_name, mode="r", pwd=password, force_zip64=True) as read_file:
        data = read_file.read()
    current_zip.close()
    file_automation_logger.info(
        f"Read zip file: {zip_file_path}"
    )
    return data


def unzip_file(
        zip_file_path: str, extract_member, extract_path: [str, None] = None, password: [str, None] = None) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="r")
    current_zip.extract(member=extract_member, path=extract_path, pwd=password)
    file_automation_logger.info(
        f"Unzip file: {zip_file_path}, "
        f"extract member: {extract_member}, "
        f"extract path: {extract_path}, "
        f"password: {password}"
    )
    current_zip.close()


def unzip_all(
        zip_file_path: str, extract_member: [str, None] = None,
        extract_path: [str, None] = None, password: [str, None] = None) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="r")
    current_zip.extractall(members=extract_member, path=extract_path, pwd=password)
    file_automation_logger.info(
        f"Unzip file: {zip_file_path}, "
        f"extract member: {extract_member}, "
        f"extract path: {extract_path}, "
        f"password: {password}"
    )
    current_zip.close()


def zip_info(zip_file_path: str) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="r")
    info_list = current_zip.infolist()
    current_zip.close()
    file_automation_logger.info(
        f"Show zip info: {zip_file_path}"
    )
    return info_list


def zip_file_info(zip_file_path: str) -> None:
    current_zip = zipfile.ZipFile(zip_file_path, mode="r")
    name_list = current_zip.namelist()
    current_zip.close()
    file_automation_logger.info(
        f"Show zip file info: {zip_file_path}"
    )
    return name_list


def set_zip_password(zip_file_path: str, password: bytes) -> None:
    current_zip = zipfile.ZipFile(zip_file_path)
    current_zip.setpassword(pwd=password)
    current_zip.close()
    file_automation_logger.info(
        f"Set zip file password, "
        f"zip file: {zip_file_path}, "
        f"zip password: {password}"
    )



import io
from io import BytesIO
from typing import Union

from googleapiclient.errors import HttpError
from googleapiclient.http import MediaIoBaseDownload

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_download_file(file_id: str, file_name: str) -> BytesIO:
    try:
        request = driver_instance.service.files().get_media(fileId=file_id)
        file = io.BytesIO()
        downloader = MediaIoBaseDownload(file, request)
        done = False
        while done is False:
            status, done = downloader.next_chunk()
            file_automation_logger.info(
                f"Download {file_name} {int(status.progress() * 100)}%."
            )
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None
    with open(file_name, "wb") as output_file:
        output_file.write(file.getbuffer())
    file_automation_logger.info(
        f"Download file: {file_id} with name: {file_name}"
    )
    return file


def drive_download_file_from_folder(folder_name: str) -> Union[dict, None]:
    try:
        files = dict()
        response = driver_instance.service.files().list(
            q=f"mimeType = 'application/vnd.google-apps.folder' and name = '{folder_name}'"
        ).execute()
        folder = response.get("files", [])[0]
        folder_id = folder.get("id")
        response = driver_instance.service.files().list(
            q=f"'{folder_id}' in parents"
        ).execute()
        for file in response.get("files", []):
            drive_download_file(file.get("id"), file.get("name"))
            files.update({file.get("name"): file.get("id")})
        file_automation_logger.info(
            f"Download all file on {folder_name} done."
        )
        return files
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None



import typing

from file_automation.local.dir.dir_process import copy_dir, create_dir, remove_dir_tree
from file_automation.local.file.file_process import copy_file, remove_file, rename_file, copy_specify_extension_file, \
    copy_all_file_to_dir
from file_automation.local.zip.zip_process import zip_dir, zip_file, zip_info, zip_file_info, set_zip_password, \
    read_zip_file, unzip_file, unzip_all
from file_automation.remote.google_drive.delete.delete_manager import drive_delete_file
from file_automation.remote.google_drive.dir.folder_manager import drive_add_folder
from file_automation.remote.google_drive.download.download_file import drive_download_file, drive_download_file_from_folder
from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.remote.google_drive.search.search_drive import \
    drive_search_all_file, drive_search_field, drive_search_file_mimetype
from file_automation.remote.google_drive.share.share_file import \
    drive_share_file_to_anyone, drive_share_file_to_domain, drive_share_file_to_user
from file_automation.remote.google_drive.upload.upload_to_driver import \
    drive_upload_dir_to_folder, drive_upload_to_folder, drive_upload_dir_to_drive, drive_upload_to_drive
from file_automation.utils.exception.exception_tags import get_bad_trigger_function, get_bad_trigger_method
from file_automation.utils.exception.exceptions import CallbackExecutorException
from file_automation.utils.logging.loggin_instance import file_automation_logger


class CallbackFunctionExecutor(object):

    def __init__(self):
        self.event_dict: dict = {
            "FA_copy_file": copy_file,
            "FA_rename_file": rename_file,
            "FA_remove_file": remove_file,
            "FA_copy_all_file_to_dir": copy_all_file_to_dir,
            "FA_copy_specify_extension_file": copy_specify_extension_file,
            "FA_copy_dir": copy_dir,
            "FA_create_dir": create_dir,
            "FA_remove_dir_tree": remove_dir_tree,
            "FA_zip_dir": zip_dir,
            "FA_zip_file": zip_file,
            "FA_zip_info": zip_info,
            "FA_zip_file_info": zip_file_info,
            "FA_set_zip_password": set_zip_password,
            "FA_unzip_file": unzip_file,
            "FA_read_zip_file": read_zip_file,
            "FA_unzip_all": unzip_all,
            "driver_instance": driver_instance,
            "search_all_file": drive_search_all_file,
            "search_field": drive_search_field,
            "search_file_mimetype": drive_search_file_mimetype,
            "upload_dir_to_folder": drive_upload_dir_to_folder,
            "upload_to_folder": drive_upload_to_folder,
            "upload_dir_to_drive": drive_upload_dir_to_drive,
            "upload_to_drive": drive_upload_to_drive,
            "add_folder": drive_add_folder,
            "share_file_to_anyone": drive_share_file_to_anyone,
            "share_file_to_domain": drive_share_file_to_domain,
            "share_file_to_user": drive_share_file_to_user,
            "delete_file": drive_delete_file,
            "download_file": drive_download_file,
            "download_file_from_folder": drive_download_file_from_folder
        }

    def callback_function(
            self,
            trigger_function_name: str,
            callback_function: typing.Callable,
            callback_function_param: [dict, None] = None,
            callback_param_method: str = "kwargs",
            **kwargs
    ) -> typing.Any:
        """
        :param trigger_function_name: what function we want to trigger only accept function in event_dict
        :param callback_function: what function we want to callback
        :param callback_function_param: callback function's param only accept dict 
        :param callback_param_method: what type param will use on callback function only accept kwargs and args
        :param kwargs: trigger_function's param
        :return: trigger_function_name return value
        """
        try:
            if trigger_function_name not in self.event_dict.keys():
                raise CallbackExecutorException(get_bad_trigger_function)
            file_automation_logger.info(f"Callback trigger {trigger_function_name} with param {kwargs}")
            execute_return_value = self.event_dict.get(trigger_function_name)(**kwargs)
            if callback_function_param is not None:
                if callback_param_method not in ["kwargs", "args"]:
                    raise CallbackExecutorException(get_bad_trigger_method)
                if callback_param_method == "kwargs":
                    callback_function(**callback_function_param)
                    file_automation_logger.info(
                        f"Callback function {callback_function} with param {callback_function_param}")
                else:
                    callback_function(*callback_function_param)
                    file_automation_logger.info(
                        f"Callback function {callback_function} with param {callback_function_param}")
            else:
                callback_function()
                file_automation_logger.info(f"Callback function {callback_function}")
            return execute_return_value
        except Exception as error:
            file_automation_logger.error(
                f"Callback function failed. {repr(error)}")


callback_executor = CallbackFunctionExecutor()



import shutil
import sys
from pathlib import Path

from file_automation.utils.exception.exceptions import FileNotExistsException, DirNotExistsException
from file_automation.utils.logging.loggin_instance import file_automation_logger


def copy_file(file_path: str, target_path: str) -> None:
    file_path = Path(file_path)
    if file_path.is_file() and file_path.exists():
        try:
            shutil.copy2(file_path, target_path)
            file_automation_logger.info(f"Copy file origin path: {file_path}, target path : {target_path}")
        except shutil.Error as error:
            file_automation_logger.error(f"Copy file failed: {repr(error)}")
    else:
        file_automation_logger.error(f"Copy file failed: {repr(FileNotExistsException)}")


def copy_specify_extension_file(file_dir_path: str, target_extension: str, target_path: str) -> None:
    file_dir_path = Path(file_dir_path)
    if file_dir_path.exists() and file_dir_path.is_dir():
        for file in file_dir_path.glob(f"**/*.{target_extension}"):
            copy_file(str(file), target_path)
            file_automation_logger.info(
                f"Copy specify extension file on dir"
                f"origin dir path: {file_dir_path}, target extension: {target_extension}, "
                f"to target path {target_path}")
    else:
        file_automation_logger.error(
            f"Copy specify extension file failed: {repr(FileNotExistsException)}")


def copy_all_file_to_dir(dir_path: str, target_dir_path: str) -> None:
    dir_path = Path(dir_path)
    target_dir_path = Path(target_dir_path)
    if dir_path.is_dir() and target_dir_path.is_dir():
        try:
            shutil.move(str(dir_path), str(target_dir_path))
            file_automation_logger.info(
                f"Copy all file to dir, "
                f"origin dir: {dir_path}, "
                f"target dir: {target_dir_path}"
            )
        except shutil.Error as error:
            file_automation_logger.error(
                f"Copy all file to dir failed, "
                f"origin dir: {dir_path}, "
                f"target dir: {target_dir_path}, "
                f"error: {repr(error)}"
            )
    else:
        print(repr(DirNotExistsException), file=sys.stderr)


def rename_file(origin_file_path, target_name: str, file_extension=None) -> None:
    origin_file_path = Path(origin_file_path)
    if origin_file_path.exists() and origin_file_path.is_dir():
        if file_extension is None:
            file_list = list(origin_file_path.glob("**/*"))
        else:
            file_list = list(origin_file_path.glob(f"**/*.{file_extension}"))
        try:
            file_index = 0
            for file in file_list:
                file.rename(Path(origin_file_path,  target_name))
                file_index = file_index + 1
                file_automation_logger.info(
                    f"Renamed file: origin file path:{origin_file_path}, with new name: {target_name}")
        except Exception as error:
            file_automation_logger.error(
                f"Rename file failed, "
                f"origin file path: {origin_file_path}, "
                f"target name: {target_name}, "
                f"file_extension: {file_extension}, "
                f"error: {repr(error)}"
            )
    else:
        file_automation_logger.error(
            f"Rename file failed, error: {repr(DirNotExistsException)}")


def remove_file(file_path: str) -> None:
    file_path = Path(file_path)
    if file_path.exists() and file_path.is_file():
        file_path.unlink(missing_ok=True)
        file_automation_logger.info(f"Remove file, file path: {file_path}")


def create_file(file_path: str, content: str) -> None:
    with open(file_path, "w+") as file:
        file.write(content)



from typing import Union

from googleapiclient.errors import HttpError

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_search_all_file() -> Union[dict, None]:
    try:
        item = dict()
        response = driver_instance.service.files().list().execute()
        for file in response.get("files", []):
            item.update({file.get("name"): file.get("id")})
        file_automation_logger.info(
            f"Search all file on drive"
        )
        return item
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_search_file_mimetype(mime_type: str) -> Union[dict, None]:
    try:
        files = dict()
        page_token = None
        while True:
            # pylint: disable=maybe-no-member
            response = driver_instance.service.files().list(
                q=f"mimeType='{mime_type}'",
                fields="nextPageToken, files(id, name)",
                pageToken=page_token).execute()
            for file in response.get("files", []):
                files.update({file.get("name"): file.get("id")})
            page_token = response.get('nextPageToken', None)
            if page_token is None:
                break
        file_automation_logger.info(
            f"Search all {mime_type} file on drive"
        )
        return files
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_search_field(field_pattern: str) -> Union[dict, None]:
    try:
        files = dict()
        response = driver_instance.service.files().list(fields=field_pattern).execute()
        for file in response.get("files", []):
            files.update({file.get("name"): file.get("id")})
        file_automation_logger.info(
            f"Search all {field_pattern}"
        )
        return files
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None




import builtins
import types
from inspect import getmembers, isbuiltin
from typing import Union, Any

from file_automation.local.dir.dir_process import copy_dir, create_dir, remove_dir_tree
from file_automation.local.file.file_process import copy_file, remove_file, rename_file, copy_specify_extension_file, \
    copy_all_file_to_dir, create_file
from file_automation.local.zip.zip_process import zip_dir, zip_file, zip_info, zip_file_info, set_zip_password, \
    read_zip_file, unzip_file, unzip_all
from file_automation.remote.google_drive.delete.delete_manager import drive_delete_file
from file_automation.remote.google_drive.dir.folder_manager import drive_add_folder
from file_automation.remote.google_drive.download.download_file import drive_download_file, \
    drive_download_file_from_folder
from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.remote.google_drive.search.search_drive import \
    drive_search_all_file, drive_search_field, drive_search_file_mimetype
from file_automation.remote.google_drive.share.share_file import \
    drive_share_file_to_anyone, drive_share_file_to_domain, drive_share_file_to_user
from file_automation.remote.google_drive.upload.upload_to_driver import \
    drive_upload_dir_to_folder, drive_upload_to_folder, drive_upload_dir_to_drive, drive_upload_to_drive
from file_automation.utils.exception.exception_tags import add_command_exception, executor_list_error, \
    action_is_null_error, cant_execute_action_error
from file_automation.utils.exception.exceptions import ExecuteActionException, AddCommandException
from file_automation.utils.json.json_file import read_action_json
from file_automation.utils.logging.loggin_instance import file_automation_logger
from file_automation.utils.package_manager.package_manager_class import package_manager
from file_automation.utils.scheduler.extend_apscheduler import scheduler_manager


class Executor(object):

    def __init__(self):
        self.event_dict: dict = {
            # File
            "FA_create_file": create_file,
            "FA_copy_file": copy_file,
            "FA_rename_file": rename_file,
            "FA_remove_file": remove_file,
            # Dir
            "FA_copy_all_file_to_dir": copy_all_file_to_dir,
            "FA_copy_specify_extension_file": copy_specify_extension_file,
            "FA_copy_dir": copy_dir,
            "FA_create_dir": create_dir,
            "FA_remove_dir_tree": remove_dir_tree,
            # Zip
            "FA_zip_dir": zip_dir,
            "FA_zip_file": zip_file,
            "FA_zip_info": zip_info,
            "FA_zip_file_info": zip_file_info,
            "FA_set_zip_password": set_zip_password,
            "FA_unzip_file": unzip_file,
            "FA_read_zip_file": read_zip_file,
            "FA_unzip_all": unzip_all,
            # Drive
            "FA_drive_later_init": driver_instance.later_init,
            "FA_drive_search_all_file": drive_search_all_file,
            "FA_drive_search_field": drive_search_field,
            "FA_drive_search_file_mimetype": drive_search_file_mimetype,
            "FA_drive_upload_dir_to_folder": drive_upload_dir_to_folder,
            "FA_drive_upload_to_folder": drive_upload_to_folder,
            "FA_drive_upload_dir_to_drive": drive_upload_dir_to_drive,
            "FA_drive_upload_to_drive": drive_upload_to_drive,
            "FA_drive_add_folder": drive_add_folder,
            "FA_drive_share_file_to_anyone": drive_share_file_to_anyone,
            "FA_drive_share_file_to_domain": drive_share_file_to_domain,
            "FA_drive_share_file_to_user": drive_share_file_to_user,
            "FA_drive_delete_file": drive_delete_file,
            "FA_drive_download_file": drive_download_file,
            "FA_drive_download_file_from_folder": drive_download_file_from_folder,
            # Execute
            "FA_execute_action": self.execute_action,
            "FA_execute_files": self.execute_files,
            "FA_add_package_to_executor": package_manager.add_package_to_executor,
            # Scheduler
            "FA_scheduler_event_trigger": self.scheduler_event_trigger,
            "FA_remove_blocking_scheduler_job": scheduler_manager.remove_blocking_job,
            "FA_remove_nonblocking_scheduler_job": scheduler_manager.remove_nonblocking_job,
            "FA_start_blocking_scheduler": scheduler_manager.start_block_scheduler,
            "FA_start_nonblocking_scheduler": scheduler_manager.start_nonblocking_scheduler,
            "FA_start_all_scheduler": scheduler_manager.start_all_scheduler,
            "FA_shutdown_blocking_scheduler": scheduler_manager.shutdown_blocking_scheduler,
            "FA_shutdown_nonblocking_scheduler": scheduler_manager.shutdown_nonblocking_scheduler,
        }
        # get all builtin function and add to event dict
        for function in getmembers(builtins, isbuiltin):
            self.event_dict.update({str(function[0]): function[1]})

    def _execute_event(self, action: list):
        event = self.event_dict.get(action[0])
        if len(action) == 2:
            if isinstance(action[1], dict):
                return event(**action[1])
            else:
                return event(*action[1])
        elif len(action) == 1:
            return event()
        else:
            raise ExecuteActionException(cant_execute_action_error + " " + str(action))

    def execute_action(self, action_list: [list, dict]) -> dict:
        """
        use to execute all action on action list(action file or program list)
        :param action_list the list include action
        for loop the list and execute action
        """
        if isinstance(action_list, dict):
            action_list: list = action_list.get("auto_control", None)
            if action_list is None:
                raise ExecuteActionException(executor_list_error)
        execute_record_dict = dict()
        try:
            if len(action_list) > 0 or isinstance(action_list, list):
                pass
            else:
                raise ExecuteActionException(action_is_null_error)
        except Exception as error:
            file_automation_logger.error(
                f"Execute {action_list} failed. {repr(error)}"
            )
        for action in action_list:
            try:
                event_response = self._execute_event(action)
                execute_record = "execute: " + str(action)
                file_automation_logger.info(
                    f"Execute {action}"
                )
                execute_record_dict.update({execute_record: event_response})
            except Exception as error:
                file_automation_logger.error(
                    f"Execute {action} failed. {repr(error)}"
                )
                execute_record = "execute: " + str(action)
                execute_record_dict.update({execute_record: repr(error)})
        for key, value in execute_record_dict.items():
            print(key, flush=True)
            print(value, flush=True)
        return execute_record_dict

    def execute_files(self, execute_files_list: list) -> list:
        """
        :param execute_files_list: list include execute files path
        :return: every execute detail as list
        """
        execute_detail_list: list = list()
        for file in execute_files_list:
            execute_detail_list.append(self.execute_action(read_action_json(file)))
        return execute_detail_list

    def scheduler_event_trigger(
            self, function: str, id: str = None, args: Union[list, tuple] = None,
            kwargs: dict = None, scheduler_type: str = "nonblocking", wait_type: str = "secondly",
            wait_value: int = 1, **trigger_args: Any) -> None:
        if scheduler_type == "nonblocking":
            scheduler_event = scheduler_manager.nonblocking_scheduler_event_dict.get(wait_type)
        else:
            scheduler_event = scheduler_manager.blocking_scheduler_event_dict.get(wait_type)
        scheduler_event(self.event_dict.get(function), id, args, kwargs, wait_value, **trigger_args)


executor = Executor()
package_manager.executor = executor


def add_command_to_executor(command_dict: dict):
    """
    :param command_dict: dict include command we want to add to event_dict
    """
    file_automation_logger.info(
        f"Add command to executor {command_dict}"
    )
    for command_name, command in command_dict.items():
        if isinstance(command, (types.MethodType, types.FunctionType)):
            executor.event_dict.update({command_name: command})
        else:
            raise AddCommandException(add_command_exception)


def execute_action(action_list: list) -> dict:
    return executor.execute_action(action_list)


def execute_files(execute_files_list: list) -> list:
    return executor.execute_files(execute_files_list)



from typing import Union, Dict

from googleapiclient.errors import HttpError

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_delete_file(file_id: str) -> Union[Dict[str, str], None]:
    try:
        file = driver_instance.service.files().delete(fileId=file_id).execute()
        file_automation_logger.info(f"Delete drive file: {file_id}")
        return file
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None



import shutil
import sys
from pathlib import Path

from file_automation.utils.exception.exceptions import DirNotExistsException
from file_automation.utils.logging.loggin_instance import file_automation_logger


def copy_dir(dir_path: str, target_dir_path: str) -> None:
    dir_path = Path(dir_path)
    target_dir_path = Path(target_dir_path)
    if dir_path.is_dir():
        try:
            shutil.copytree(dir_path, target_dir_path, dirs_exist_ok=True)
            file_automation_logger.info(f"Copy dir {dir_path}")
        except shutil.Error as error:
            file_automation_logger.error(f"Copy dir {dir_path} failed: {repr(error)}")
    else:
        file_automation_logger.error(f"Copy dir {dir_path} failed: {repr(DirNotExistsException)}")


def remove_dir_tree(dir_path: str) -> None:
    dir_path = Path(dir_path)
    if dir_path.is_dir():
        try:
            shutil.rmtree(dir_path)
            file_automation_logger.info(f"Remove dir tree {dir_path}")
        except shutil.Error as error:
            file_automation_logger.error(f"Remove dir tree {dir_path} error: {repr(error)}")


def rename_dir(origin_dir_path, target_dir: str) -> None:
    origin_dir_path = Path(origin_dir_path)
    if origin_dir_path.exists() and origin_dir_path.is_dir():
        try:
            Path.rename(origin_dir_path, target_dir)
            file_automation_logger.info(
                f"Rename dir origin dir path: {origin_dir_path}, target dir path: {target_dir}")
        except Exception as error:
            file_automation_logger.error(
                f"Rename dir error:  {repr(error)}, "
                f"Rename dir origin dir path: {origin_dir_path}, "
                f"target dir path: {target_dir}")
    else:
        file_automation_logger.error(
            f"Rename dir error:  {repr(DirNotExistsException)}, "
            f"Rename dir origin dir path: {origin_dir_path}, "
            f"target dir path: {target_dir}")


def create_dir(dir_path: str) -> None:
    dir_path = Path(dir_path)
    dir_path.mkdir(exist_ok=True)
    file_automation_logger.info(f"Create dir {dir_path}")



from pathlib import Path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from file_automation.utils.logging.loggin_instance import file_automation_logger


class GoogleDrive(object):

    def __init__(self):
        self.google_drive_instance = None
        self.creds = None
        self.service = None
        self.scopes = ["https://www.googleapis.com/auth/drive"]

    def later_init(self, token_path: str, credentials_path: str):
        token_path = Path(token_path)
        credentials_path = Path(credentials_path)
        creds = None
        # The file token.json stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if token_path.exists():
            file_automation_logger.info(
                f"Token exists try to load."
            )
            creds = Credentials.from_authorized_user_file(str(token_path), self.scopes)
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    str(credentials_path), self.scopes)
                creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open(str(token_path), 'w') as token:
                token.write(creds.to_json())
        try:
            self.service = build('drive', 'v3', credentials=creds)
            file_automation_logger.info(
                f"Loading service successfully."
            )
        except HttpError as error:
            file_automation_logger.error(
                f"Delete file failed,"
                f"error: {error}"
            )


driver_instance = GoogleDrive()




from typing import Union

from googleapiclient.errors import HttpError

from file_automation.remote.google_drive.driver_instance import driver_instance
from file_automation.utils.logging.loggin_instance import file_automation_logger


def drive_share_file_to_user(
        file_id: str, user: str, user_role: str = "writer") -> Union[dict, None]:
    try:
        service = driver_instance.service
        user_permission = {
            "type": "user",
            "role": user_role,
            "emailAddress": user
        }
        file_automation_logger.info(
            f"Share file: {file_id}, "
            f"to user: {user}, "
            f"with user role: {user_role}"
        )
        return service.permissions().create(
            fileId=file_id,
            body=user_permission,
            fields='id', ).execute()
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_share_file_to_anyone(file_id: str, share_role: str = "reader") -> Union[dict, None]:
    try:
        service = driver_instance.service
        user_permission = {
            "type": "anyone",
            "value": "anyone",
            "role": share_role
        }
        file_automation_logger.info(
            f"Share file to anyone file: {file_id} with role: {share_role}"
        )
        return service.permissions().create(
            fileId=file_id,
            body=user_permission,
            fields='id', ).execute()
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None


def drive_share_file_to_domain(
        file_id: str, domain: str, domain_role: str = "reader") -> Union[dict, None]:
    try:
        service = driver_instance.service
        domain_permission = {
            "type": "domain",
            "role": domain_role,
            "domain": domain
        }
        file_automation_logger.info(
            f"Share file to domain: {domain}, "
            f"with domain role: {domain_role}"
        )
        return service.permissions().create(
            fileId=file_id,
            body=domain_permission,
            fields='id', ).execute()
    except HttpError as error:
        file_automation_logger.error(
            f"Delete file failed,"
            f"error: {error}"
        )
        return None



class FileAutomationException(Exception):
    pass


class FileNotExistsException(FileAutomationException):
    pass


class DirNotExistsException(FileAutomationException):
    pass


class ZIPGetWrongFileException(FileAutomationException):
    pass


class CallbackExecutorException(FileAutomationException):
    pass


class ExecuteActionException(FileAutomationException):
    pass


class AddCommandException(FileAutomationException):
    pass


class JsonActionException(FileAutomationException):
    pass



token_is_exist: str = "token file is exists"
# Callback executor
get_bad_trigger_method: str = "get bad trigger method, only accept kwargs and args"
get_bad_trigger_function: str = "get bad trigger function only accept function in event_dict"
# add command
add_command_exception: str = "command value type should be as method or function"
# executor
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# json tag
cant_execute_action_error: str = "cant execute action"
cant_generate_json_report: str = "can't generate json report"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
action_is_null_error: str = "json action is null"
